;; Generated package description from wisitoken-grammar-mode.el  -*- no-byte-compile: t -*-
(define-package "wisitoken-grammar-mode" "1.3.0" "Major mode for editing WisiToken grammar files" '((wisi "4.2.2") (emacs "25.3") (mmm-mode "0.5.7")) :commit "2d58879cea26b8a3b573d084d149dea94f93dfb8" :authors '(("Stephen Leake" . "stephen_leake@stephe-leake.org")) :maintainer '("Stephen Leake" . "stephen_leake@stephe-leake.org") :keywords '("languages") :url "https://www.nongnu.org/ada-mode/")
